@@ -1,7 +1,7 @@
 #include "SWFReader.h"
 #include "SWFTag.h"
 #include "SWFAction.h"
-#include <string.h>
+#include <cstring>
 
 namespace SWF {
 
@@ -21,6 +21,7 @@ bool Reader::getData( void *dst, size_t len ) {
 	} else {
 		memcpy( dst, &data[pos], len );
 		pos += len;
+		return true;
 	}
 }
 
@@ -74,6 +75,41 @@ uint64_t Reader::getInt64() {
 	return( val );
 }
 
+uint32_t Reader::getU30() {
+	uint32_t r = 0;
+	unsigned char c;
+
+	for( int i = 0; i < 5; i++ ) {
+		c = data[pos++];
+		r |= (c & 0x7F) << (7 * i);
+		
+		if( !(c & 0x80) ) {
+			return r;
+		}
+
+		if( pos > length ) {
+			err = SWFR_EOF;
+			pos = length+1;
+			return 0;
+		}
+	}
+
+	return r;
+}
+
+int Reader::getS24() {
+	if( pos+3 > length ) {
+		err = SWFR_EOF;
+		pos = length+1;
+		return 0;
+	}
+
+	int r = data[pos++];
+	r += data[pos++]<<8;
+	r += ((signed char)data[pos++])<<16;
+	return r;
+}
+
 float Reader::getFloat() {
     /*
 	if( pos+4 > length ) {
@@ -125,6 +161,77 @@ double Reader::getDouble() {
     return u.d;
 }
 
+double Reader::getDouble2() {
+	if( pos+8 > length ) {
+		err = SWFR_EOF;
+		pos = length+1;
+		return 0;
+	}
+	
+	union {
+		double d;
+		char c[8];
+	} u;
+
+	
+	// FIXME x86-centric?
+	for( int i = 0; i < 8; i++ ) {
+		u.c[i] =  data[pos++];
+	}
+
+	return u.d;
+}
+
+float Reader::getHalf() {
+	uint16_t r = getWord();
+	
+	int sign = (r & 0x8000) >> 15;
+	int exp = (r & 0x7C00) >> 10;
+	int man = (r & 0x3FF);
+
+	if(exp == 0) {
+		if(man != 0) {
+			while(!(man & 0x400)) {
+				man <<= 1;
+				exp -= 1;
+			}
+			exp += 1 - 15 + 127;
+			man &= 0x3FF;
+		}
+	} else if(exp == 0x1F) {
+		exp = 0xFF;
+	} else {
+		exp += -15 + 127;
+	}
+
+    union {
+        float f;
+        uint32_t ul;
+    } u;
+
+	u.ul = (sign << 31) | (exp << 23) | (man << 13);
+	return u.f;
+}
+
+unsigned int Reader::getEncodedU32() {
+    
+    unsigned int result = data[pos++];
+    if (!(result & 0x00000080)) return result;
+
+    result = (result & 0x0000007f) | data[pos++] << 7;
+    if (!(result & 0x00004000)) return result;
+
+    result = (result & 0x00003fff) | data[pos++] << 14;
+    if (!(result & 0x00200000)) return result;
+
+    result = (result & 0x001fffff) | data[pos++] << 21;
+    if (!(result & 0x10000000)) return result;
+
+    result = (result & 0x0fffffff) | data[pos++] << 28;
+    return result;
+
+}
+
 double Reader::getFixed( int bytesize, int exp ) {
 	/* putFixed/getFixed are deprecated: they implicitly to byteAlign */
 	
@@ -143,9 +250,12 @@ double Reader::getFixed( int bytesize, int exp ) {
 
 char *Reader::getString() {
 	byteAlign();
-	int len = strlen((const char *)&data[pos]);
-	pos += len+1;
-	return strdup( (const char *)&data[pos-(len+1)] );
+	const char *src = (const char *)&data[pos];
+	size_t len = strlen(src) + 1;
+	char *dst = new char[len];
+	strcpy(dst, src);
+	pos += len;
+	return( dst );
 }
 
 char *Reader::getPString() {
@@ -157,6 +267,15 @@ char *Reader::getPString() {
 	return( dst );
 }
 
+char *Reader::getPStringU30() {
+	byteAlign();
+	uint32_t len = getU30();
+	char *dst = new char[len+1];
+	getData( dst, len );
+	dst[len]=0;
+	return( dst );
+}
+
 long Reader::getNBitInt( int n, bool is_signed ) {
 	// FIXME: unefficient, maybe incorrect!
 	int orig_n = n;
