@@ -1,7 +1,7 @@
 #ifndef SWF_Reader_H
 #define SWF_Reader_H
 
-#include <stdio.h>
+#include <cstdio>
 #include <stdint.h>
 #include "SWFBasic.h"
 
@@ -21,11 +21,18 @@ class Reader {
 		uint16_t getWord();
 		uint32_t getInt();
 		uint64_t getInt64();
+		uint32_t getU30();
+		int getS24();
 		float getFloat();
 		double getDouble();
+		double getDouble2();
+		float getHalf();
 		double getFixed( int bytesize, int decimals );
 		char *getString();
 		char *getPString();
+		char *getPStringU30();
+		
+		unsigned int getEncodedU32();	
 	
 		// this one cares about the current bit position
 		long getNBitInt( int n, bool is_signed = false );
@@ -38,7 +45,7 @@ class Reader {
 			buf = bits = 0; }
 				
 		int getError() { return err; }
-			
+
 		const unsigned char *getRawData() { return data; }
 		int getPosition() { return pos; }
 		int getLength() { return length; }
