@@ -28,7 +28,7 @@ class LineTo : public ShapeItem {
 		LineTo();
 		virtual bool parse( Reader *r, int end, Context *ctx );
 		virtual void dump( int indent, Context *ctx );
-		virtual size_t getSize( Context *ctx );
+		virtual size_t calcSize( Context *ctx, int start_at );
 		virtual void write( Writer *w, Context *ctx );
 		virtual void writeXML( xmlNodePtr node, Context *ctx );
 		virtual void parseXML( xmlNodePtr node, Context *ctx );
