@@ -1,7 +1,7 @@
 #include <libexslt/exslt.h>
 #include "SWFFile.h"
 #include "swft.h"
-#include "xslt.h"
+#include "../src/xslt/xslt.h"
 #include <cstdlib>
 #include <cstring>
 #include <sys/types.h>
@@ -54,6 +54,7 @@ void usage() {
 		"\n"
 		"<option>s are:\n"
 		"    -h print this help and quit\n"
+		"    --version print the version number and quit\n"
 		"    -v verbose output\n"
 		"    -V extra-verbose debugging output\n"
 		"    -d dump SWF data when loaded (for debugging)\n"
@@ -61,10 +62,14 @@ void usage() {
 		"           default: UTF-8).\n"
 		"    -n deactivate libxml network access\n"
 		"\n"
-		"Please report bugs at http://bugs.launchpad.net/swfmill/+filebug\n\n"
+		"Please report bugs at https://github.com/djcsdy/swfmill/issues\n\n"
 	);
 }
 
+void printVersion() {
+	fprintf(stderr, PACKAGE_NAME" "PACKAGE_VERSION"\n");
+}
+
 // helper func
 xsltStylesheetPtr xsltParseStylesheetMemory( const char *buffer, int size ) {
 	xmlDocPtr doc = xmlParseMemory( buffer, size );
@@ -341,7 +346,7 @@ fail:
 void swfmill_create_library( xmlNodePtr lib, const char *filename );
 
 void swfmill_create_library_dir( xmlNodePtr lib, const char *dir ) {
-	char tmp[256];
+	char tmp[1024];
 	
 	struct dirent *e;
 	DIR *d = opendir( dir );
@@ -352,7 +357,7 @@ void swfmill_create_library_dir( xmlNodePtr lib, const char *dir ) {
 		if( e==NULL ) break;
 		if( e->d_name[0] == '.' ) continue; // no hidden files
 		
-		snprintf( tmp, 1023, "%s/%s", dir, e->d_name );
+		snprintf( tmp, sizeof(tmp), "%s/%s", dir, e->d_name );
 		swfmill_create_library( lib, tmp );
 	}
 }
@@ -422,6 +427,11 @@ int main( int argc, char *argv[] ) {
 // parse args
 	int i=1;
 	for( ; i<argc && command==NULL; i++ ) {
+		if (strncmp(argv[i], "--version", 10) == 0) {
+			printVersion();
+			return 0;
+		}
+		
 		if( argv[i][0] == '-' ) {
 			int swallow=0;
 			for( int j=1; j<strlen(argv[i]); j++ ) {
@@ -444,7 +454,7 @@ int main( int argc, char *argv[] ) {
 					case 'h':
 					case '?':
 						usage();
-						goto fail;
+						return 0;
 						break;
 					case 'e':
 						++swallow;
@@ -456,7 +466,7 @@ int main( int argc, char *argv[] ) {
 						}
 						break;
 					default:
-						fprintf(stderr,"ERROR: unknown option %c\n",argv[i][j]);
+						fprintf(stderr,"ERROR: unknown option -%c\n",argv[i][j]);
 						usage();
 						goto fail;
 				}
