@@ -14,6 +14,8 @@
 #include FT_OUTLINE_H
 #include "SWFShapeMaker.h"
 
+#include <freetype/tttables.h>
+
 using namespace SWF;
 
 #define TMP_STRLEN 0xff
@@ -37,7 +39,12 @@ int conicTo( const FT_Vector *control, const FT_Vector *to, void *shaper ) {
 }
 
 int cubicTo( const FT_Vector *control1, const FT_Vector *control2, const FT_Vector *to, void *shaper ) {
-	fprintf(stderr,"ERROR: cubic beziers in fonts are not yet implemented.\n");
+
+	((ShapeMaker*)shaper)->cubicTo (
+		control1->x, control1->y,
+		control2->x, control2->y,
+		to->x, to->y
+	);
 
 	return 0;
 }
@@ -61,6 +68,27 @@ int compareGlyphs( const void *a, const void *b ) {
 	return( _a - _b );
 }
 
+void getVerticalMetrics (FT_Face face, int &ascender, int &descender, int &lineGap)
+{
+	unsigned char *pOS2 = (unsigned char *) FT_Get_Sfnt_Table(face, ft_sfnt_os2);
+	if (pOS2)
+	{
+		ascender = (short) (pOS2 [76] | (pOS2 [77] << 8));
+		descender = (short) (pOS2 [78] | (pOS2 [79] << 8));
+		lineGap = (short) (pOS2 [74] | (pOS2 [75] << 8));
+	}
+	else
+	{
+		unsigned char *pHhea = (unsigned char *) FT_Get_Sfnt_Table(face, ft_sfnt_hhea);
+		if (pHhea)
+		{
+			ascender = (short) (pHhea [4] | (pHhea [5] << 8));
+			descender = -(short) (pHhea [6] | (pHhea [7] << 8));
+			lineGap = (short) (pHhea [8] | (pHhea [9] << 8));
+		}
+	}
+}
+
 void importDefineFont2( DefineFont2 *tag, const char *filename, const char *fontname, const xmlChar *glyphs_xml, Context *ctx, swft_ctx *swftctx, int offset ) {
 	FT_Library swfft_library;
 	FT_Face face;
@@ -81,6 +109,8 @@ void importDefineFont2( DefineFont2 *tag, const char *filename, const char *font
 	GlyphShape *shape;
 	int nGlyphs, glyph;
 	
+	const int SCALING_FACTOR = 1024;
+
 	if( FT_Init_FreeType( &swfft_library ) ) {
 		fprintf( stderr, "WARNING: could not initialize FreeType\n" );
 		goto fail;
@@ -93,10 +123,12 @@ void importDefineFont2( DefineFont2 *tag, const char *filename, const char *font
 	}
 
 	if( face->num_faces > 1 ) {
-		fprintf( stderr, "WARNING: %s contains %i faces, but only the first is imported.\n", filename, face->num_faces );
+		fprintf(stderr,
+				"WARNING: %s contains %li faces, but only the first is "
+				"imported.\n", filename, face->num_faces);
 	}
 
-	FT_Set_Char_Size(face, (1024<<6), (1024<<6), 72, 72);
+	FT_Set_Pixel_Sizes(face, SCALING_FACTOR, SCALING_FACTOR);
 
 	// count availably glyphs, yes we have to load them to check if they're empty, sorry.
 	nGlyphs = 0;
@@ -104,15 +136,17 @@ void importDefineFont2( DefineFont2 *tag, const char *filename, const char *font
 		if( (character = FT_Get_First_Char( face, &glyph_index )) != 0 ) nGlyphs++;
 		while( (character = FT_Get_Next_Char( face, character, &glyph_index )) != 0 ) {
 			if( FT_Load_Glyph( face, glyph_index, FT_LOAD_NO_BITMAP ) ) {
-				fprintf( stderr, "WARNING: cannot load glyph %i ('%c') from %s.\n", character, character, filename );
+				fprintf( stderr,
+						"WARNING: cannot load glyph %lu ('%c') from "
+						"%s.\n", character, character, filename );
 				goto fail;
 			}
 			if( face->glyph->format != FT_GLYPH_FORMAT_OUTLINE ) {
 				fprintf( stderr, "WARNING: %s seems to be a bitmap font.\n", filename );
 				goto fail;
 			}
-			//if( !emptyGlyph( face, character ) ) 
-				nGlyphs++;
+			
+			nGlyphs++;
 		}
 		
 		glyphs = new int[nGlyphs+1];
@@ -147,13 +181,14 @@ void importDefineFont2( DefineFont2 *tag, const char *filename, const char *font
 	
 	tag->sethasLayout( 1 );
 	
-#define SCALING_FACTOR ((int)1024)
+	{
+		int ascender, descender, lineGap;
+		getVerticalMetrics (face, ascender, descender, lineGap);
+		tag->setascent (ascender * SCALING_FACTOR / face->units_per_EM);
+		tag->setdescent (descender * SCALING_FACTOR / face->units_per_EM);
+		tag->setleading (lineGap * SCALING_FACTOR / face->units_per_EM);
+	}
 
-	tag->setascent( 1+((SCALING_FACTOR * face->ascender) / face->units_per_EM) );
-	tag->setdescent( 1+((SCALING_FACTOR * labs(face->descender)) / face->units_per_EM) );
-	tag->setleading( 0 ) ;// (1+(SCALING_FACTOR * face->ascender) / face->units_per_EM) + (1+(SCALING_FACTOR * labs(face->descender)) / face->units_per_EM) );
-			//1+(SCALING_FACTOR * face->height) / face->units_per_EM );
-	
 	tag->setwideGlyphOffsets( 1 );
 	tag->setwideMap( 1 );
 	
@@ -175,7 +210,8 @@ void importDefineFont2( DefineFont2 *tag, const char *filename, const char *font
 		glyph_index = FT_Get_Char_Index( face, character );
 		
 		if( FT_Load_Glyph( face, glyph_index, FT_LOAD_NO_BITMAP ) ) {
-			fprintf( stderr, "WARNING: cannot load glyph %i ('%c') from %s.\n", character, character, filename );
+			fprintf( stderr, "WARNING: cannot load glyph %lu ('%c') "
+					"from %s.\n", character, character, filename );
 			goto fail;
 		}
 
@@ -201,13 +237,13 @@ void importDefineFont2( DefineFont2 *tag, const char *filename, const char *font
 		shaper.finish();
 
 		Rectangle *r = new Rectangle();
-        
+
 		Rect rect = shaper.getBounds();
 		r->setleft( (int)rect.left );
 		r->settop( (int)rect.top );
 		r->setright( (int)rect.right );
 		r->setbottom( (int)rect.bottom );
-        
+
 		r->setbits( SWFMaxBitsNeeded( true, 3, r->gettop(), r->getright(), r->getbottom() ) );
 		bounds->append(r);
 	}
@@ -269,6 +305,9 @@ void importDefineFont3( DefineFont3 *tag, const char *filename, const char *font
 	
 	GlyphShape *shape;
 	int nGlyphs, glyph;
+
+	const int SCALING_FACTOR = 1024 * 20;
+
 	
 	if( FT_Init_FreeType( &swfft_library ) ) {
 		fprintf( stderr, "WARNING: could not initialize FreeType\n" );
@@ -282,10 +321,11 @@ void importDefineFont3( DefineFont3 *tag, const char *filename, const char *font
 	}
 
 	if( face->num_faces > 1 ) {
-		fprintf( stderr, "WARNING: %s contains %i faces, but only the first is imported.\n", filename, face->num_faces );
+		fprintf( stderr, "WARNING: %s contains %li faces, but only the "
+				"first is imported.\n", filename, face->num_faces );
 	}
 
-	FT_Set_Char_Size(face, (1024<<6) * 20, (1024<<6) * 20, 72, 72);
+	FT_Set_Char_Size(face, SCALING_FACTOR << 6, SCALING_FACTOR << 6, 72, 72);
 
 	// count availably glyphs, yes we have to load them to check if they're empty, sorry.
 	nGlyphs = 0;
@@ -293,7 +333,9 @@ void importDefineFont3( DefineFont3 *tag, const char *filename, const char *font
 		if( (character = FT_Get_First_Char( face, &glyph_index )) != 0 ) nGlyphs++;
 		while( (character = FT_Get_Next_Char( face, character, &glyph_index )) != 0 ) {
 			if( FT_Load_Glyph( face, glyph_index, FT_LOAD_NO_BITMAP ) ) {
-				fprintf( stderr, "WARNING: cannot load glyph %i ('%c') from %s.\n", character, character, filename );
+				fprintf(stderr,
+						"WARNING: cannot load glyph %lu ('%c') "
+						"from %s.\n", character, character, filename);
 				goto fail;
 			}
 			if( face->glyph->format != FT_GLYPH_FORMAT_OUTLINE ) {
@@ -336,12 +378,13 @@ void importDefineFont3( DefineFont3 *tag, const char *filename, const char *font
 	
 	tag->sethasLayout( 1 );
 	
-#define SCALING_FACTOR ((int)1024)
-
-	tag->setascent( (1+((SCALING_FACTOR * face->ascender) / face->units_per_EM)) * 20 );
-	tag->setdescent( (1+((SCALING_FACTOR * labs(face->descender)) / face->units_per_EM)) * 20 );
-	tag->setleading( 0 ) ;// (1+(SCALING_FACTOR * face->ascender) / face->units_per_EM) + (1+(SCALING_FACTOR * labs(face->descender)) / face->units_per_EM) );
-			//1+(SCALING_FACTOR * face->height) / face->units_per_EM );
+	{
+		int ascender, descender, lineGap;
+		getVerticalMetrics (face, ascender, descender, lineGap);
+		tag->setascent (ascender * SCALING_FACTOR / face->units_per_EM);
+		tag->setdescent (descender * SCALING_FACTOR / face->units_per_EM);
+		tag->setleading (lineGap * SCALING_FACTOR / face->units_per_EM);
+	}
 	
 	tag->setwideGlyphOffsets( 1 );
 	tag->setwideMap( 1 );
@@ -364,7 +407,9 @@ void importDefineFont3( DefineFont3 *tag, const char *filename, const char *font
 		glyph_index = FT_Get_Char_Index( face, character );
 		
 		if( FT_Load_Glyph( face, glyph_index, FT_LOAD_NO_BITMAP ) ) {
-			fprintf( stderr, "WARNING: cannot load glyph %i ('%c') from %s.\n", character, character, filename );
+			fprintf(stderr,
+					"WARNING: cannot load glyph %lu ('%c') from %s.\n",
+					character, character, filename);
 			goto fail;
 		}
 
@@ -390,13 +435,13 @@ void importDefineFont3( DefineFont3 *tag, const char *filename, const char *font
 		shaper.finish();
 
 		Rectangle *r = new Rectangle();
-        
+
 		Rect rect = shaper.getBounds();
 		r->setleft( (int)rect.left );
 		r->settop( (int)rect.top );
 		r->setright( (int)rect.right );
 		r->setbottom( (int)rect.bottom );
-        
+
 		r->setbits( SWFMaxBitsNeeded( true, 3, r->gettop(), r->getright(), r->getbottom() ) );
 		bounds->append(r);
 	}
@@ -442,7 +487,7 @@ fail:
 
 void swft_import_ttf( xmlXPathParserContextPtr ctx, int nargs ) {
 	xsltTransformContextPtr tctx;
-	xmlChar *filename;
+	char *filename;
 	xsltDocumentPtr xsltdoc;
 	xmlDocPtr doc = NULL;
 	xmlNodePtr node;
@@ -452,6 +497,9 @@ void swft_import_ttf( xmlXPathParserContextPtr ctx, int nargs ) {
 	xmlChar *glyphs = NULL;
 	int offset;
 	double movieVersion;
+	bool quiet = true;
+	xmlXPathObjectPtr quietObj = NULL;
+	swft_ctx *c = NULL;
 	
 	const char *fontname = NULL;
 	
@@ -473,14 +521,14 @@ void swft_import_ttf( xmlXPathParserContextPtr ctx, int nargs ) {
 	}
 
 	movieVersion = xmlXPathPopNumber(ctx);
-	filename = swft_get_filename( xmlXPathPopString(ctx) );
-	if( xmlXPathCheckError(ctx) )
-		return;
+	filename = swft_get_filename(xmlXPathPopString(ctx), ctx->context->doc->URL);
+	if (xmlXPathCheckError(ctx)) {
+		goto fail;
+	}
 	
 	tctx = xsltXPathGetTransformContext(ctx);
 	
-	bool quiet = true;
-	xmlXPathObjectPtr quietObj = xsltVariableLookup( tctx, (const xmlChar*)"quiet", NULL );
+	quietObj = xsltVariableLookup( tctx, (const xmlChar*)"quiet", NULL );
 	if( quietObj && quietObj->stringval ) { quiet = !strcmp("true",(const char*)quietObj->stringval ); };
 	swfctx.quiet = quiet;
 		
@@ -488,9 +536,7 @@ void swft_import_ttf( xmlXPathParserContextPtr ctx, int nargs ) {
 	doc->xmlRootNode = xmlNewDocNode( doc, NULL, (const xmlChar*)"ttf", NULL );
 	node = doc->xmlRootNode;
 	
-	//swft_addFileName( node, (const char *)filename );
-	
-	swft_ctx *c = (swft_ctx*)xsltGetExtData( xsltXPathGetTransformContext(ctx), SWFT_NAMESPACE );
+	c = (swft_ctx*)xsltGetExtData( xsltXPathGetTransformContext(ctx), SWFT_NAMESPACE );
 
 	// create the font tag	
 	if( movieVersion >= 8 ) {
@@ -503,17 +549,15 @@ void swft_import_ttf( xmlXPathParserContextPtr ctx, int nargs ) {
 		tag->writeXML( node, &swfctx );
 	}
 	
-/*
-	snprintf(tmp,TMP_STRLEN,"%i", 5);
-	xmlSetProp( node, (const xmlChar *)"format", (const xmlChar *)&tmp );
-*/	
 	if( glyphs )
 		xmlFree( glyphs );
 
 	valuePush( ctx, xmlXPathNewNodeSet( (xmlNodePtr)doc ) );
-	return;
+	goto end;
 	
 fail:
 	fprintf( stderr, "WARNING: could not import %s\n", filename );
-	return;
+
+end:
+	delete filename;
 }
